# Generated by roxygen2: do not edit by hand

S3method("$",pdata.frame)
S3method("$<-",pdata.frame)
S3method("[",pdata.frame)
S3method("[[",pdata.frame)
S3method(Between,default)
S3method(Between,matrix)
S3method(Between,pseries)
S3method(Complex,pseries)
S3method(Math,pseries)
S3method(Ops,pseries)
S3method(Sum,default)
S3method(Sum,matrix)
S3method(Sum,pseries)
S3method(Within,default)
S3method(Within,matrix)
S3method(Within,pseries)
S3method(alias,pdata.frame)
S3method(alias,plm)
S3method(as.data.frame,pdata.frame)
S3method(as.list,pdata.frame)
S3method(as.matrix,pseries)
S3method(between,default)
S3method(between,matrix)
S3method(between,pseries)
S3method(coef,panelmodel)
S3method(coef,pgmm)
S3method(coef,summary.plm.list)
S3method(detect.lindep,data.frame)
S3method(detect.lindep,matrix)
S3method(detect.lindep,plm)
S3method(deviance,panelmodel)
S3method(df.residual,panelmodel)
S3method(diff,pseries)
S3method(ercomp,formula)
S3method(ercomp,pdata.frame)
S3method(ercomp,plm)
S3method(fitted,panelmodel)
S3method(fitted,plm)
S3method(fixef,pggls)
S3method(fixef,plm)
S3method(formula,dynformula)
S3method(formula,pdata.frame)
S3method(formula,plm)
S3method(has.intercept,Formula)
S3method(has.intercept,default)
S3method(has.intercept,formula)
S3method(has.intercept,panelmodel)
S3method(has.intercept,plm)
S3method(index,panelmodel)
S3method(index,pdata.frame)
S3method(index,pindex)
S3method(index,pseries)
S3method(is.pbalanced,data.frame)
S3method(is.pbalanced,default)
S3method(is.pbalanced,panelmodel)
S3method(is.pbalanced,pcce)
S3method(is.pbalanced,pdata.frame)
S3method(is.pbalanced,pggls)
S3method(is.pbalanced,pgmm)
S3method(is.pbalanced,pmg)
S3method(is.pbalanced,pseries)
S3method(is.pconsecutive,data.frame)
S3method(is.pconsecutive,default)
S3method(is.pconsecutive,panelmodel)
S3method(is.pconsecutive,pdata.frame)
S3method(is.pconsecutive,pseries)
S3method(lag,pseries)
S3method(lead,pseries)
S3method(make.dummies,data.frame)
S3method(make.dummies,default)
S3method(make.dummies,pdata.frame)
S3method(make.pbalanced,data.frame)
S3method(make.pbalanced,pdata.frame)
S3method(make.pbalanced,pseries)
S3method(make.pconsecutive,data.frame)
S3method(make.pconsecutive,pdata.frame)
S3method(make.pconsecutive,pseries)
S3method(model.frame,pdata.frame)
S3method(model.matrix,pcce)
S3method(model.matrix,pdata.frame)
S3method(model.matrix,plm)
S3method(mtest,pgmm)
S3method(nobs,panelmodel)
S3method(nobs,pgmm)
S3method(pFtest,formula)
S3method(pFtest,plm)
S3method(pbgtest,formula)
S3method(pbgtest,panelmodel)
S3method(pbltest,formula)
S3method(pbltest,plm)
S3method(pbnftest,formula)
S3method(pbnftest,panelmodel)
S3method(pbsytest,formula)
S3method(pbsytest,panelmodel)
S3method(pcdtest,formula)
S3method(pcdtest,panelmodel)
S3method(pcdtest,pseries)
S3method(pdim,data.frame)
S3method(pdim,default)
S3method(pdim,panelmodel)
S3method(pdim,pcce)
S3method(pdim,pdata.frame)
S3method(pdim,pggls)
S3method(pdim,pgmm)
S3method(pdim,pmg)
S3method(pdim,pseries)
S3method(pdwtest,formula)
S3method(pdwtest,panelmodel)
S3method(phtest,formula)
S3method(phtest,panelmodel)
S3method(plmtest,formula)
S3method(plmtest,plm)
S3method(plot,plm)
S3method(plot,pseries)
S3method(plot,summary.pseries)
S3method(pmodel.response,data.frame)
S3method(pmodel.response,formula)
S3method(pmodel.response,pcce)
S3method(pmodel.response,plm)
S3method(pooltest,formula)
S3method(pooltest,plm)
S3method(predict,plm)
S3method(print,dynformula)
S3method(print,ercomp)
S3method(print,fixef)
S3method(print,panelmodel)
S3method(print,pdata.frame)
S3method(print,pdim)
S3method(print,phansitest)
S3method(print,piest)
S3method(print,plm.list)
S3method(print,pseries)
S3method(print,purtest)
S3method(print,pvar)
S3method(print,summary.fixef)
S3method(print,summary.pcce)
S3method(print,summary.pggls)
S3method(print,summary.pgmm)
S3method(print,summary.pht)
S3method(print,summary.piest)
S3method(print,summary.plm)
S3method(print,summary.plm.list)
S3method(print,summary.pmg)
S3method(print,summary.pseries)
S3method(print,summary.purtest)
S3method(print,summary.pvcm)
S3method(punbalancedness,data.frame)
S3method(punbalancedness,default)
S3method(punbalancedness,panelmodel)
S3method(punbalancedness,pdata.frame)
S3method(pvar,data.frame)
S3method(pvar,default)
S3method(pvar,matrix)
S3method(pvar,pdata.frame)
S3method(pvar,pseries)
S3method(pwaldtest,default)
S3method(pwaldtest,pgmm)
S3method(pwaldtest,plm)
S3method(pwaldtest,pvcm)
S3method(pwartest,formula)
S3method(pwartest,panelmodel)
S3method(pwfdtest,formula)
S3method(pwfdtest,panelmodel)
S3method(pwtest,formula)
S3method(pwtest,panelmodel)
S3method(ranef,plm)
S3method(residuals,panelmodel)
S3method(residuals,pcce)
S3method(residuals,pggls)
S3method(residuals,plm)
S3method(residuals,pmg)
S3method(sargan,pgmm)
S3method(summary,fixef)
S3method(summary,pcce)
S3method(summary,pggls)
S3method(summary,pgmm)
S3method(summary,pht)
S3method(summary,piest)
S3method(summary,plm)
S3method(summary,plm.list)
S3method(summary,pmg)
S3method(summary,pseries)
S3method(summary,purtest)
S3method(summary,pvcm)
S3method(terms,panelmodel)
S3method(tss, default)
S3method(tss, plm)
S3method(update,panelmodel)
S3method(vcov,panelmodel)
S3method(vcovBK,pcce)
S3method(vcovBK,pggls)
S3method(vcovBK,plm)
S3method(vcovDC,pcce)
S3method(vcovDC,pggls)
S3method(vcovDC,plm)
S3method(vcovG,pcce)
S3method(vcovG,pggls)
S3method(vcovG,plm)
S3method(vcovHC,pcce)
S3method(vcovHC,pggls)
S3method(vcovHC,pgmm)
S3method(vcovHC,plm)
S3method(vcovNW,pcce)
S3method(vcovNW,pggls)
S3method(vcovNW,plm)
S3method(vcovSCC,pcce)
S3method(vcovSCC,pggls)
S3method(vcovSCC,plm)
S3method(within_intercept,plm)
export(Between)
export(Sum)
export(Within)
export(aneweytest)
export(between)
export(cipstest)
export(cortab)
export(detect.lindep)
export(dynformula)
export(ercomp)
export(fixef)
export(has.intercept)
export(index)
export(is.pbalanced)
export(is.pconsecutive)
export(is.pseries)
export(lag)
export(lead)
export(make.dummies)
export(make.pbalanced)
export(make.pconsecutive)
export(maxLik)
export(mtest)
export(nobs)
export(pFtest)
export(pbgtest)
export(pbltest)
export(pbnftest)
export(pbsytest)
export(pcce)
export(pcdtest)
export(pdata.frame)
export(pdim)
export(pdwtest)
export(pggls)
export(pgmm)
export(pgrangertest)
export(phansitest)
export(pht)
export(phtest)
export(piest)
export(pldv)
export(plm)
export(plm.data)
export(plmtest)
export(pmg)
export(pmodel.response)
export(pooltest)
export(pseriesfy)
export(punbalancedness)
export(purtest)
export(pvar)
export(pvcm)
export(pvcovHC)
export(pwaldtest)
export(pwartest)
export(pwfdtest)
export(pwtest)
export(r.squared)
export(ranef)
export(sargan)
export(vcovBK)
export(vcovDC)
export(vcovG)
export(vcovHC)
export(vcovNW)
export(vcovSCC)
export(within_intercept)
if(getRversion() >= "3.6.0") {S3method(dplyr::arrange, pdata.frame)}
if(getRversion() >= "3.6.0") {S3method(dplyr::arrange, pindex)}
import(Formula)
importFrom(MASS,ginv)
importFrom(Rdpack,reprompt)
importFrom(bdsmatrix,bdsmatrix)
importFrom(collapse,GRP)
importFrom(collapse,GRPN)
importFrom(collapse,dapply)
importFrom(collapse,fbetween)
importFrom(collapse,fdroplevels)
importFrom(collapse,fhdwithin)
importFrom(collapse,fwithin)
importFrom(collapse,gsplit)
importFrom(collapse,rsplit)
importFrom(grDevices,heat.colors)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(lattice,xyplot)
importFrom(lmtest,bgtest)
importFrom(lmtest,dwtest)
importFrom(maxLik,maxLik)
importFrom(nlme,fixef)
importFrom(nlme,lme)
importFrom(nlme,ranef)
importFrom(sandwich,vcovHC)
importFrom(stats,.lm.fit)
importFrom(stats,alias)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,contr.treatment)
importFrom(stats,cor)
importFrom(stats,delete.response)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,lag)
importFrom(stats,lm)
importFrom(stats,lm.fit)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,reshape)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(zoo,index)
