Programmer's Learning Machine (PLM)
=========

The Programmer's Learning Machine (PLM) is a platform dedicated to computer
programming education. This generic platform offers support to teachers for
creating programming microworlds suitable to teaching courses. It features an
integrated and graphical environment, providing a short feedback loop to
students in order to improve the effectiveness of the autonomous learning
process.

For more information, subscribe to our
[announcement](https://sympa.inria.fr/sympa/subscribe/plm-announce)
mailing list (low trafic), and 
[visit the PLM web site](http://www.loria.fr/~quinson/Teaching/PLM/).

[![Build Status](https://travis-ci.org/oster/PLM.svg)](https://travis-ci.org/oster/PLM)

## License

PLM is licensed under the GNU General Public License 3.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <http://www.gnu.org/licenses/>.

The documentation, tutorial and source code are intended as a community
resource and you can basically use, copy and improve them however you want.
Included works are subject to their respective licenses.
