Rapport d'activité du 24/11 au 28/11
==============================

Activité de la semaine
------------------------------------

* Implémentation d'un script permettant de récupérer l'ensemble des messages de commit de [PLM-server](https://github.com/mquinson/PLM-server), servant de base à [Harvester](https://github.com/mquinson/PLM-server/blob/master/script/git/browse/Harvester.java).
* Mise en place et tests d'environnement de développement d'une application Web utilisant [Play Framework](https://www.playframework.com/) pour le back-end et [Dart]() pour le front-end.
* Suivi du [tutoriel d'AngularDart](https://angulardart.org/) et implémentation d'une [application de démonstration](https://github.com/MatthieuNICOLAS/play-with-dart)

Problèmes rencontrés
-------------------------------------

* Difficultés à mettre en place l'environnement de développement
	* Aucun IDE parmi *Eclipse*, *Intellij IDEA* ou  *Dart Editor*  ne supporte nativement **Play Framework**  et **Dart**)
	* Des plugins existent pour permettrent à *Eclipse* ou *Intellij IDEA* de supporter **Dart**, mais certaines fonctionnalités décrites ([Dart-to-JS compiler](https://www.dartlang.org/tools/webstorm/#dart-to-js-compiler) par exemple) semblent absentes.
	* Des conflits peuvent se produire entre Chromium et Dartium, empêchant ce dernier de fonctionner.
* Release récente de la version 1.0 d'**AngularDart**: certains [exemples](https://angulardart.org/demo/) de code n'ont pas encore été mis à jour.

Objectifs pour la semaine suivante
---------------------------------------------------------
* S'assurer que la procédure de mise en place de l'environnement de développement décrite [ici](https://github.com/MatthieuNICOLAS/play-with-dart/blob/master/README.md) est complète.
* &Eacute;tudier comment déployer [l'application](https://github.com/MatthieuNICOLAS/play-with-dart) au format *standalone* et la rendre facile d'utilisation.
	* Possibles pistes [ici](https://www.playframework.com/documentation/2.4.x/ProductionDist) et [là](http://scalatron.github.io/).
* Implémenter une application basée sur **Play With Dart** créant une instance de **PLM** et permettant à un utilisateur d'interagir avec depuis une page Web.
* Revoir l'état de la branche command-view et poursuivre son développement.